@@ -1,7 +1,7 @@
 /*
    Directory hotlist -- for the Midnight Commander
 
-   Copyright (C) 1994-2015
+   Copyright (C) 1994-2017
    Free Software Foundation, Inc.
 
    Written by:
@@ -71,10 +71,6 @@
 #define UX 3
 #define UY 2
 
-#define BX UX
-#define BY (LINES - 6)
-
-#define LABELS          3
 #define B_ADD_CURRENT   B_USER
 #define B_REMOVE        (B_USER + 1)
 #define B_NEW_GROUP     (B_USER + 2)
@@ -290,12 +286,12 @@ fill_listbox (WListbox * list)
                 g_string_truncate (buff, 0);
                 g_string_append (buff, "->");
                 g_string_append (buff, current->label);
-                listbox_add_item (list, LISTBOX_APPEND_AT_END, 0, buff->str, current);
+                listbox_add_item (list, LISTBOX_APPEND_AT_END, 0, buff->str, current, FALSE);
             }
             break;
         case HL_TYPE_DOTDOT:
         case HL_TYPE_ENTRY:
-            listbox_add_item (list, LISTBOX_APPEND_AT_END, 0, current->label, current);
+            listbox_add_item (list, LISTBOX_APPEND_AT_END, 0, current->label, current, FALSE);
         default:
             break;
         }
@@ -328,17 +324,15 @@ unlink_entry (struct hotlist *entry)
 static void
 add_name_to_list (const char *path)
 {
-    listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, path, 0);
+    listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, path, NULL, FALSE);
 }
 #endif /* !ENABLE_VFS */
 
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-hotlist_button_callback (WButton * button, int action)
+hotlist_run_cmd (int action)
 {
-    (void) button;
-
     switch (action)
     {
     case B_MOVE:
@@ -476,7 +470,8 @@ hotlist_button_callback (WButton * button, int action)
 
     case B_REFRESH_VFS:
         listbox_remove_list (l_hotlist);
-        listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, mc_config_get_home_dir (), 0);
+        listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, mc_config_get_home_dir (), NULL,
+                          FALSE);
         vfs_fill_names (add_name_to_list);
         return 0;
 #endif /* ENABLE_VFS */
@@ -488,6 +483,19 @@ hotlist_button_callback (WButton * button, int action)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static int
+hotlist_button_callback (WButton * button, int action)
+{
+    int ret;
+
+    (void) button;
+    ret = hotlist_run_cmd (action);
+    update_path_name ();
+    return ret;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static inline cb_ret_t
 hotlist_handle_key (WDialog * h, int key)
 {
@@ -564,16 +572,43 @@ hotlist_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
 
     switch (msg)
     {
+    case MSG_INIT:
+    case MSG_NOTIFY:           /* MSG_NOTIFY is fired by the listbox to tell us the item has changed. */
+        update_path_name ();
+        return MSG_HANDLED;
+
     case MSG_UNHANDLED_KEY:
         return hotlist_handle_key (h, parm);
 
     case MSG_POST_KEY:
-        dlg_select_widget (h == hotlist_dlg ? l_hotlist : l_movelist);
-        /* always stay on hotlist */
-        /* fall through */
+        /*
+         * The code here has two purposes:
+         *
+         * (1) Always stay on the hotlist.
+         *
+         * Activating a button using its hotkey (and even pressing ENTER, as
+         * there's a "default button") moves the focus to the button. But we
+         * want to stay on the hotlist, to be able to use the usual keys (up,
+         * down, etc.). So we do `widget_select (lst)`.
+         *
+         * (2) Refresh the hotlist.
+         *
+         * We may have run a command that changed the contents of the list.
+         * We therefore need to refresh it. So we do `widget_redraw (lst)`.
+         */
+        {
+            Widget *lst;
 
-    case MSG_INIT:
-        update_path_name ();
+            lst = WIDGET (h == hotlist_dlg ? l_hotlist : l_movelist);
+
+            /* widget_select() already redraws the widget, but since it's a
+             * no-op if the widget is already selected ("focused"), we have
+             * to call widget_redraw() separately. */
+            if (!widget_get_state (lst, WST_FOCUSED))
+                widget_select (lst);
+            else
+                widget_redraw (lst);
+        }
         return MSG_HANDLED;
 
     case MSG_RESIZE:
@@ -684,7 +719,7 @@ init_i18n_stuff (int list_type, int cols)
         (len[0])--;
         (len[1])--;
 
-        cols = max (cols, max (len[0], len[1]));
+        cols = MAX (cols, MAX (len[0], len[1]));
 
         /* arrange buttons */
         for (i = 0; i < hotlist_but_num; i++)
@@ -729,7 +764,7 @@ init_hotlist (hotlist_t list_type)
     cols = init_i18n_stuff (list_type, COLS - 6);
 
     hotlist_state.expanded =
-        mc_config_get_int (mc_main_config, "HotlistConfig", "expanded_view_of_groups", 0);
+        mc_config_get_int (mc_global.main_config, "HotlistConfig", "expanded_view_of_groups", 0);
 
 #ifdef ENABLE_VFS
     if (list_type == LIST_VFSLIST)
@@ -746,8 +781,8 @@ init_hotlist (hotlist_t list_type)
     }
 
     hotlist_dlg =
-        dlg_create (TRUE, 0, 0, lines, cols, dialog_colors, hotlist_callback, NULL, help_node,
-                    title, DLG_CENTER);
+        dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, hotlist_callback,
+                    NULL, help_node, title);
 
     y = UY;
     hotlist_group = groupbox_new (y, UX, lines - 10 + dh, cols - 2 * UX, _("Top level group"));
@@ -762,7 +797,8 @@ init_hotlist (hotlist_t list_type)
 #ifdef ENABLE_VFS
     if (list_type == LIST_VFSLIST)
     {
-        listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, mc_config_get_home_dir (), 0);
+        listbox_add_item (l_hotlist, LISTBOX_APPEND_AT_END, 0, mc_config_get_home_dir (), NULL,
+                          FALSE);
         vfs_fill_names (add_name_to_list);
     }
     else
@@ -791,7 +827,7 @@ init_hotlist (hotlist_t list_type)
                                             hotlist_but[i].text, hotlist_button_callback),
                                 hotlist_but[i].pos_flags, NULL);
 
-    dlg_select_widget (l_hotlist);
+    widget_select (WIDGET (l_hotlist));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -813,8 +849,8 @@ init_movelist (struct hotlist *item)
     hdr = g_strdup_printf (_("Moving %s"), item->label);
 
     movelist_dlg =
-        dlg_create (TRUE, 0, 0, lines, cols, dialog_colors, hotlist_callback, NULL, "[Hotlist]",
-                    hdr, DLG_CENTER);
+        dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, hotlist_callback,
+                    NULL, "[Hotlist]", hdr);
 
     g_free (hdr);
 
@@ -842,7 +878,7 @@ init_movelist (struct hotlist *item)
                                             hotlist_but[i].text, hotlist_button_callback),
                                 hotlist_but[i].pos_flags, NULL);
 
-    dlg_select_widget (l_movelist);
+    widget_select (WIDGET (l_movelist));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -856,8 +892,9 @@ hotlist_done (void)
 {
     dlg_destroy (hotlist_dlg);
     l_hotlist = NULL;
-    if (FALSE)
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+#if 0
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+#endif
     repaint_screen ();
 }
 
@@ -947,11 +984,11 @@ add2hotlist (char *label, char *directory, enum HotListType type, listbox_append
             char *lbl;
 
             lbl = g_strconcat ("->", new->label, (char *) NULL);
-            listbox_add_item (l_hotlist, pos, 0, lbl, new);
+            listbox_add_item (l_hotlist, pos, 0, lbl, new, FALSE);
             g_free (lbl);
         }
         else
-            listbox_add_item (l_hotlist, pos, 0, new->label, new);
+            listbox_add_item (l_hotlist, pos, 0, new->label, new, FALSE);
         listbox_select_entry (l_hotlist, l_hotlist->pos);
     }
 
@@ -1164,21 +1201,21 @@ load_group (struct hotlist *grp)
 
     group_section = find_group_section (grp);
 
-    keys = mc_config_get_keys (mc_main_config, group_section, NULL);
+    keys = mc_config_get_keys (mc_global.main_config, group_section, NULL);
 
     current_group = grp;
 
     for (profile_keys = keys; *profile_keys != NULL; profile_keys++)
-        add2hotlist (mc_config_get_string (mc_main_config, group_section, *profile_keys, ""),
+        add2hotlist (mc_config_get_string (mc_global.main_config, group_section, *profile_keys, ""),
                      g_strdup (*profile_keys), HL_TYPE_GROUP, LISTBOX_APPEND_AT_END);
 
     g_free (group_section);
     g_strfreev (keys);
 
-    keys = mc_config_get_keys (mc_main_config, grp->directory, NULL);
+    keys = mc_config_get_keys (mc_global.main_config, grp->directory, NULL);
 
     for (profile_keys = keys; *profile_keys != NULL; profile_keys++)
-        add2hotlist (mc_config_get_string (mc_main_config, group_section, *profile_keys, ""),
+        add2hotlist (mc_config_get_string (mc_global.main_config, group_section, *profile_keys, ""),
                      g_strdup (*profile_keys), HL_TYPE_ENTRY, LISTBOX_APPEND_AT_END);
 
     g_strfreev (keys);
@@ -1389,20 +1426,20 @@ clean_up_hotlist_groups (const char *section)
     char *grp_section;
 
     grp_section = g_strconcat (section, ".Group", (char *) NULL);
-    if (mc_config_has_group (mc_main_config, section))
-        mc_config_del_group (mc_main_config, section);
+    if (mc_config_has_group (mc_global.main_config, section))
+        mc_config_del_group (mc_global.main_config, section);
 
-    if (mc_config_has_group (mc_main_config, grp_section))
+    if (mc_config_has_group (mc_global.main_config, grp_section))
     {
         char **profile_keys, **keys;
 
-        keys = mc_config_get_keys (mc_main_config, grp_section, NULL);
+        keys = mc_config_get_keys (mc_global.main_config, grp_section, NULL);
 
         for (profile_keys = keys; *profile_keys != NULL; profile_keys++)
             clean_up_hotlist_groups (*profile_keys);
 
         g_strfreev (keys);
-        mc_config_del_group (mc_main_config, grp_section);
+        mc_config_del_group (mc_global.main_config, grp_section);
     }
     g_free (grp_section);
 }
@@ -1469,10 +1506,10 @@ load_hotlist (void)
         GError *mcerror = NULL;
 
         clean_up_hotlist_groups ("Hotlist");
-        if (!mc_config_save_file (mc_main_config, &mcerror))
-            setup_save_config_show_error (mc_main_config->ini_path, &mcerror);
+        if (!mc_config_save_file (mc_global.main_config, &mcerror))
+            setup_save_config_show_error (mc_global.main_config->ini_path, &mcerror);
 
-        mc_error_message (&mcerror);
+        mc_error_message (&mcerror, NULL);
     }
 
     stat (hotlist_file_name, &stat_buf);
@@ -1538,6 +1575,8 @@ do { \
         case HL_TYPE_DOTDOT:
             /* do nothing */
             break;
+        default:
+            break;
         }
 }
 
