@@ -1,7 +1,7 @@
 /*
    Print features specific for this build
 
-   Copyright (C) 2000-2014
+   Copyright (C) 2000-2017
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -33,6 +33,7 @@
 #include "lib/global.h"
 #include "lib/fileloc.h"
 #include "lib/mcconfig.h"
+#include "lib/util.h"           /* mc_get_profile_root() */
 
 #include "src/textconf.h"
 
@@ -180,7 +181,8 @@ show_version (void)
 void
 show_datadirs_extended (void)
 {
-    (void) printf ("%s %s\n", _("Root directory:"), mc_config_get_home_dir ());
+    (void) printf ("%s %s\n", _("Home directory:"), mc_config_get_home_dir ());
+    (void) printf ("%s %s\n", _("Profile root directory:"), mc_get_profile_root ());
     (void) puts ("");
 
     PRINTF_GROUP (_("System data"));
@@ -193,10 +195,10 @@ show_datadirs_extended (void)
 #if defined ENABLE_VFS_EXTFS || defined ENABLE_VFS_FISH
     PRINTF_SECTION (_("VFS plugins and scripts:"), LIBEXECDIR);
 #ifdef ENABLE_VFS_EXTFS
-    PRINTF2 ("extfs.d:", LIBEXECDIR, MC_EXTFS_DIR "/");
+    PRINTF2 ("extfs.d:", LIBEXECDIR, MC_EXTFS_DIR PATH_SEP_STR);
 #endif
 #ifdef ENABLE_VFS_FISH
-    PRINTF2 ("fish:", LIBEXECDIR, FISH_PREFIX "/");
+    PRINTF2 ("fish:", LIBEXECDIR, FISH_PREFIX PATH_SEP_STR);
 #endif
 #endif /* ENABLE_VFS_EXTFS || defiined ENABLE_VFS_FISH */
     (void) puts ("");
@@ -205,12 +207,12 @@ show_datadirs_extended (void)
 
     PRINTF_SECTION2 (_("Config directory:"), mc_config_get_path ());
     PRINTF_SECTION2 (_("Data directory:"), mc_config_get_data_path ());
-    PRINTF ("skins:", mc_config_get_data_path (), MC_SKINS_SUBDIR "/");
+    PRINTF ("skins:", mc_config_get_data_path (), MC_SKINS_SUBDIR PATH_SEP_STR);
 #ifdef ENABLE_VFS_EXTFS
-    PRINTF ("extfs.d:", mc_config_get_data_path (), MC_EXTFS_DIR "/");
+    PRINTF ("extfs.d:", mc_config_get_data_path (), MC_EXTFS_DIR PATH_SEP_STR);
 #endif
 #ifdef ENABLE_VFS_FISH
-    PRINTF ("fish:", mc_config_get_data_path (), FISH_PREFIX "/");
+    PRINTF ("fish:", mc_config_get_data_path (), FISH_PREFIX PATH_SEP_STR);
 #endif
 #ifdef USE_INTERNAL_EDIT
     PRINTF ("mcedit macros:", mc_config_get_data_path (), MC_MACRO_FILE);
